@@ -9,6 +9,7 @@
 #include <glib/gprintf.h>
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
 #include <uim/gettext.h>
@@ -70,6 +71,10 @@ static void show_preedit(GtkIMContext *ic, GtkWidget *preedit_label);
 
 static void helper_disconnect_cb(void);
 static void helper_read_cb(gpointer p, int fd, GdkInputCondition c);
+static GdkFilterReturn
+toplevel_window_candidate_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data);
+
+static void layout_candidate(IMUIMContext *uic);
 
 static void
 im_uim_parse_helper_str(char *str);
@@ -338,7 +343,8 @@ update_status_window(IMUIMContext *uic)
   }
 }
 
-void focus_in(GtkIMContext *ic)
+static void
+focus_in(GtkIMContext *ic)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
   IMUIMContext *cc;
@@ -358,6 +364,13 @@ void focus_in(GtkIMContext *ic)
     }
   }
 
+  if (uic->win && uic->toplevel) {
+    gdk_window_add_filter(uic->toplevel,
+			  toplevel_window_candidate_cb,
+			  uic);
+       
+    }
+
   uim_helper_client_focus_in(uic->uc);
 
   update_status_window(uic);
@@ -365,8 +378,8 @@ void focus_in(GtkIMContext *ic)
     gtk_widget_show(uic->stat_win);
   }
 }
-
-void focus_out(GtkIMContext *ic)
+static void
+focus_out(GtkIMContext *ic)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
@@ -376,6 +389,10 @@ void focus_out(GtkIMContext *ic)
 
   uim_helper_client_focus_out(uic->uc);
 
+  if (uic->win) {
+    gdk_window_remove_filter(uic->toplevel, toplevel_window_candidate_cb, uic);
+  }
+
   if(uic->cwin.top_win){
     gtk_widget_hide(uic->cwin.top_win);
   }
@@ -498,22 +515,6 @@ set_client_window(GtkIMContext *ic, GdkWindow *w)
   if(w)
     toplevel = gdk_window_get_toplevel(w);
 
-  if ( uic->win != w ) {
-    if (uic->win) {
-      /*      gdk_window_remove_filter(uic->toplevel, resize_window_cb, uic);*/
-      gdk_window_remove_filter(uic->toplevel, toplevel_window_candidate_cb, uic);
-    }
-    if (w && toplevel) {
-      /*      gdk_window_add_filter(toplevel,
-			     resize_window_cb,
-			     uic);*/
-       
-       gdk_window_add_filter(toplevel,
-			     toplevel_window_candidate_cb,
-			     uic);
-       
-    }
-  }
   uic->win = w;
   uic->toplevel = toplevel;
   update_status_window(uic);
@@ -639,7 +640,7 @@ update_prop_list_cb(void *ptr, char *str)
 {
   GString *tmp = g_string_new("");
   g_string_printf(tmp, "prop_list_update\ncharset=UTF-8\n%s", str);
- 
+
   uim_helper_send_message(im_uim_fd, tmp->str);
   g_string_free(tmp, TRUE);
 }
@@ -702,7 +703,9 @@ init_candidate_win(IMUIMContext *uic)
   gtk_box_pack_start(GTK_BOX(vbox), cwin->clist, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), numlabel, FALSE, FALSE, 0);
 
-  gtk_container_add(GTK_CONTAINER(cwin->top_win),vbox);
+  gtk_window_set_default_size(cwin->top_win, 80, 0);
+
+  gtk_container_add(GTK_CONTAINER(cwin->top_win), vbox);
   {
     gint x,y,w,h,d;
     gdk_window_get_geometry(uic->win, &x, &y,&w,&h,&d);
@@ -710,6 +713,8 @@ init_candidate_win(IMUIMContext *uic)
     gtk_window_move(GTK_WINDOW(cwin->top_win), x , y+h);
   }
 
+
+
   gtk_clist_set_column_auto_resize( GTK_CLIST(cwin->clist), 0, TRUE );
   gtk_clist_set_column_auto_resize( GTK_CLIST(cwin->clist), 1, TRUE );
 
@@ -955,7 +960,6 @@ im_uim_parse_helper_str(char *str)
   tmp = g_strsplit(str, "\n\n", 0);
 
   while(tmp[i]){
-    /*    fprintf(stderr,"tmp:%d:%s\n",i,tmp[i]);*/
   
     if(focused_context) {
       if(strcmp("prop_list_get", tmp[i]) == 0){
@@ -982,35 +986,21 @@ im_uim_parse_helper_str(char *str)
 static void
 helper_read_cb(gpointer p, int fd, GdkInputCondition c)
 {
-  /*  uim_helper_read_proc();*/
-  static char buf[1024];
-  int i,rc;
+  char *tmp;
+
+  tmp = uim_helper_read_proc(fd);
 
   if(!result)
     result = strdup("");
 
-  while((i = uim_helpler_fd_readable(fd)) > 0) {
-
-    rc = read(fd, buf, sizeof(buf)-1);
-    
-    if(rc == 0) {
-      gdk_input_remove(read_tag);
-      return;
-    }
-
-    buf[rc] = '\0';
-    result = (char*)realloc(result, strlen(result) + strlen(buf)+1);
-  
-    strcat(result, buf);
-  }
+  result = (char*)realloc(result, strlen(result) + strlen(tmp)+1);
+  strcat(result, tmp);
 
   if(uim_helper_str_terminated(result)) {
     im_uim_parse_helper_str(result);
     free(result);
-    result = NULL;
+    result = strdup("");
   }
-  
-
 }
 
 static void
