@@ -1,3 +1,5 @@
+;; Anthy is a Japanese input method
+;;
 ;;
 (require "japanese.scm")
 (require "generic-key.scm")
@@ -712,12 +714,12 @@
 
 (define anthy-update-prop-label
   (lambda (ac)
-    (let* ((id (anthy-context-ac-id ac))
+    (let ((id (anthy-context-ac-id ac))
 	   (str ""))
       (set! str
 	    (if (anthy-context-on ac)
 		"\tҤ餬\n"
-		"a\tľ\n"))  
+		"a\tľ\n"))
       (set! str (string-append str "\tϥ⡼\n"))
       (im-update-prop-label id str))))
   
@@ -742,10 +744,15 @@
 	    (if (anthy-context-on ac)
 		"\tҤ餬\n"
 		"a\tľ\n")))
-      (set! str (string-append str "leaf\t\tҤ餬\tҤ餬ʤǤ\tprop_anthy_hiragana\nleaf\t\t\tʤϤǤޤ\tprop_anthy_katakana\nleaf\tA\tľ\t쥯ȤǤ\tprop_anthy_direct\nbranch\t\t\nleaf\t\t޻\tdesc\tprop_anthy_roma\nleaf\t\t\t\tdescription\tprop_anthy_kana\n"))
-    (im-update-prop-list id str)
-    
-    )))
+      (set! str (string-append
+		 str
+		 "leaf\t\tҤ餬\tҤ餬ʤǤ\tprop_anthy_hiragana\n"
+		 "leaf\t\t\tʤϤǤޤ\tprop_anthy_katakana\n"
+		 "leaf\tA\tľ\t쥯ȤǤ\tprop_anthy_direct\n"
+		 "branch\t\t\nleaf\t\t޻\tdesc\tprop_anthy_roma\n"
+		 "leaf\t\t\t\tdescription\tprop_anthy_kana\n"))
+      (im-update-prop-list id str)
+      )))
 
 (register-im
  'anthy
