@@ -1,6 +1,7 @@
 ;; SKK is a Japanese input method
 ;;
 ;; SKKϤϲξ֤ǹ
+;; Following is list of SKK input state
 ;;  ľ direct
 ;;   kanji
 ;;  Ѵ converting
@@ -24,7 +25,7 @@
 (define skk-use-candidate-window? #t)
 (define skk-candidate-op-count 0)
 (define skk-use-recursive-learning? #f)
-;; key
+;; key defs
 (define skk-latin-key
   (lambda (key key-state)
     (or
@@ -392,6 +393,7 @@
 	   (skk-context-set-state! sc 'skk-state-wide-latin)
 	   (rk-flush rkc)
 	   (skk-update-mode id sc)
+	   (skk-update-prop-label sc id)
 	   #f)
 	 #t)
      (if (skk-latin-key key key-state)
@@ -399,12 +401,14 @@
 	   (skk-context-set-state! sc 'skk-state-latin)
 	   (rk-flush rkc)
 	   (skk-update-mode id sc)
+	   (skk-update-prop-label sc id)
 	   #f)
 	 #t)
      (if (skk-kana-toggle key key-state)
 	 (begin 
 	   (skk-context-kana-toggle sc)
 	   (skk-update-mode id sc)
+	   (skk-update-prop-label sc id)
 	   #f)
 	 #t)
      #t)))
@@ -493,6 +497,7 @@
 	   (begin 
 	     (skk-context-kana-toggle sc)
 	     (skk-update-mode id sc)
+	     (skk-update-prop-label sc id)
 	     #f)
 	   #t)
        (if (skk-begin-conv-key key key-state)
@@ -747,7 +752,8 @@
        (skk-on-key key key-state)
        (begin
 	 (skk-context-set-state! sc 'skk-state-direct)
-	 (skk-update-mode (context-id c) sc))
+	 (skk-update-mode (context-id c) sc)
+	 (skk-update-prop-label sc id))
        (skk-commit-raw sc (context-id c) key key-state))
       ())))
 
@@ -761,6 +767,7 @@
 	   (begin
 	     (skk-flush sc)
 	     (skk-update-mode id sc)
+	     (skk-update-prop-label sc id)
 	     #f)
 	   #t)
        (if (modifier-key-mask key-state)
@@ -809,7 +816,8 @@
       (im-pushback-mode-list id "")
       (im-pushback-mode-list id "ѱѿ")
       (im-update-mode-list id)
-      (im-update-mode id 0))))
+      (im-update-mode id 0)
+      (skk-update-prop-list id))))
 
 (define skk-press-key-handler
   (lambda (id key state)
@@ -863,6 +871,72 @@
       (skk-context-set-nth! sc idx)
       (skk-update-preedit  id sc))))
 
+
+(define skk-prop-handler
+  (lambda (id message)
+    (let* ((c (find-context id))
+	   (sc (context-data c)))
+      (skk-flush sc)
+      (skk-update-preedit sc id)
+    (if (string-equal? message
+		       "prop_skk_hiragana")
+	(begin
+	  (im-update-mode id 1)
+	  (skk-context-set-state! sc #t)))
+    (if (string-equal? message
+		       "prop_skk_katakana")
+	(begin
+	  (im-update-mode id 2)
+	  (skk-context-set-on! sc #t)))
+    (if (string-equal? message
+		       "prop_skk_latin")
+	(begin
+	  (im-update-mode id 0)
+	  (skk-context-set-state! sc 'skk-state-latin)))
+    (if (string-equal? message
+		       "prop_skk_wide_latin")
+	(begin
+	  (im-update-mode id 3)
+	  (skk-context-set-state! sc 'skk-state-wide-latin)))
+    (skk-update-prop-label sc id))))
+
+
+(define skk-update-prop-label
+  (lambda (sc id)
+    (let* ((state (skk-context-state sc))
+	   (kana? (skk-context-kana-mode sc))
+	   (str ""))
+      (if kana?
+	  (set! str "\tҤ餬\n")
+	  (set! str "\t\n"))
+      (if (= state 'skk-state-latin)
+	  (set! str "S\tľ\n"))
+      (if (= state 'skk-state-wide-latin)
+	  (set! str "\tѱѿ\n"))
+      (im-update-prop-label id str))))
+
+(define skk-update-prop-list
+  (lambda (id) 
+    (let* ((c (find-context id))
+	   (sc (context-data c))
+	   (state (skk-context-state sc))
+	   (kana? (skk-context-kana-mode sc))
+	   (str "branch\t"))
+      (if kana?
+	  (set! str "\tҤ餬\n")
+	  (set! str "\t\n"))
+      (if (= state 'skk-state-latin)
+	  (set! str "S\tľ\n"))
+      (if (= state 'skk-state-wide-latin)
+	  (set! str "\tѱѿ\n"))
+      (set! str (string-append "branch\t" str
+			       "leaf\t\tҤ餬\tҤ餬ʥ⡼\tprop_skk_hiragana\n"
+			       "leaf\t\t\tʥ⡼\tprop_skk_katakana\n"
+			       "leaf\tS\tľ\tľϥ⡼\tprop_skk_latin\n"
+			       "leaf\t\tѱѿ\tѱѿ⡼\tprop_skk_wide_latin\n"))
+      (im-update-prop-list id str)
+      )))
+
 (register-im
  'skk
  "ja"
@@ -876,4 +950,5 @@
  skk-reset-handler
  skk-get-candidate-handler
  skk-set-candidate-index-handler
- nil)
+ skk-prop-handler
+ )
