@@ -17,6 +17,10 @@
 
 extern LISP sym_t;
 
+/*
+ * cand : candidate
+ */
+
 
 /*
  * |C0|C1| .. |Cnr_real_cands| ..              |Cnr_cands|
@@ -443,7 +447,7 @@ skk_search_line_from_cache(struct dic_info *di, char *s, char okuri_head)
 
 
 static struct skk_cand_array *
-find_candidate_array(struct dic_info *di, char *s,
+find_cand_array(struct dic_info *di, char *s,
 		     char okuri_head, char *okuri)
 {
   struct skk_line *sl, *sl_file;
@@ -463,6 +467,7 @@ find_candidate_array(struct dic_info *di, char *s,
   ca = find_candidate_array_from_line(sl, okuri, 0);
 
   if (!ca->is_used) {
+    /* incorporate okuri-nasi entry */
     merge_candidate_array(sl, ca);
     ca->is_used = 1;
     if (!from_file) {
@@ -476,7 +481,8 @@ find_candidate_array(struct dic_info *di, char *s,
 }
 
 static struct skk_cand_array *
-find_cand_array_lisp(LISP head_, LISP okuri_head_, LISP okuri_)
+find_cand_array_lisp(LISP head_, LISP okuri_head_, LISP okuri_,
+		     int create_if_not_found)
 {
   int n;
   char o;
@@ -495,7 +501,7 @@ find_cand_array_lisp(LISP head_, LISP okuri_head_, LISP okuri_)
     o = os[0];
   }
 
-  ca = find_candidate_array(skk_dic, hs, o, okuri);
+  ca = find_cand_array(skk_dic, hs, o, okuri);
   free(okuri);
   return ca;
 }
@@ -504,7 +510,8 @@ find_cand_array_lisp(LISP head_, LISP okuri_head_, LISP okuri_)
 static LISP
 skk_get_entry(LISP head_, LISP okuri_head_, LISP okuri_)
 {
-  struct skk_cand_array *ca = find_cand_array_lisp(head_, okuri_head_, okuri_);
+  struct skk_cand_array *ca;
+  ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 0);
   if (ca) {
     return sym_t;
   }
@@ -519,7 +526,7 @@ skk_get_nth_candidate(LISP nth_, LISP head_,
   struct skk_cand_array *ca;
   char *str;
 
-  ca = find_cand_array_lisp(head_, okuri_head_, okuri_);
+  ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 0);
   n = get_c_long(nth_);
   if (ca && ca->nr_cands > n) {
     str = ca->cands[n];
@@ -534,7 +541,7 @@ skk_get_nr_candidates(LISP head_, LISP okuri_head_, LISP okuri_)
 {
   struct skk_cand_array *ca;
   int n = 0;
-  ca = find_cand_array_lisp(head_, okuri_head_, okuri_);
+  ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 0);
 
   if (ca) {
     n = ca->nr_cands;
@@ -581,7 +588,7 @@ skk_commit_candidate(LISP head_, LISP okuri_head_,
     return NIL;
   }
 
-  ca = find_cand_array_lisp(head_, okuri_head_, okuri_);
+  ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 0);
   if (!ca || ca->nr_cands <= nth) {
     return NIL;
   }
@@ -590,7 +597,7 @@ skk_commit_candidate(LISP head_, LISP okuri_head_,
   reorder_candidate(ca, str);
 
   if (okuri_ != NIL) {
-    ca = find_cand_array_lisp(head_, okuri_head_, NIL);
+    ca = find_cand_array_lisp(head_, okuri_head_, NIL, 0);
     if (!ca || ca->nr_cands <= nth) {
       return NIL;
     }
@@ -602,9 +609,30 @@ skk_commit_candidate(LISP head_, LISP okuri_head_,
   return NIL;
 }
 
+static void
+learn_word_to_cand_array(struct skk_cand_array *ca, char *word)
+{
+}
+
 static LISP
 skk_learn_word(LISP head_, LISP okuri_head_, LISP okuri_, LISP word_)
 {
+  struct skk_cand_array *ca;
+  char *word;
+
+  word = uim_get_c_string(word_);
+
+  ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 0);
+  if (ca) {
+    learn_word_to_cand_array(ca, word);
+  }
+  if (okuri_ != NIL) {
+    ca = find_cand_array_lisp(head_, okuri_head_, NIL, 0);
+    if (ca) {
+      learn_word_to_cand_array(ca, word);
+    }
+  }
+  free(word);
   return NIL;
 }
 
