@@ -13,6 +13,8 @@
 static int uim_fd;
 static void (*uim_disconnect_cb)(void);
 
+static void uim_helper_client_focus(uim_context uc, int flg);
+
 #define BUFFER_SIZE 1024
 static char uim_help_buf[BUFFER_SIZE];
 /*Common buffer for some functions's temporary buffer.
@@ -90,8 +92,27 @@ uim_helper_client_get_prop_list(void)
 }
 
 
-void
-uim_helper_read_proc(void)
+char *
+uim_helper_read_proc(int fd)
 {
-  // currently do nothing
+  int i,rc;  
+  char buf[1024];
+  char *tmp = strdup("");
+
+  while((i = uim_helpler_fd_readable(fd)) > 0) {
+    
+    rc = read(fd, buf, sizeof(buf)-1);
+    
+    if(rc == 0) {
+      uim_disconnect_cb();
+      return tmp;
+    }
+    
+    buf[rc] = '\0';
+    tmp = (char*)realloc(tmp, strlen(tmp) + strlen(buf)+1);
+    
+    strcat(tmp, buf);
+  }
+  
+  return tmp;
 }
