@@ -24,6 +24,7 @@ static uim_context context_array[CONTEXT_ARRAY_SIZE];
 struct uim_im *uim_im_array;
 int uim_nr_im;
 static int uim_initialized;
+int uim_siod_fatal;
 
 void
 uim_set_preedit_cb(uim_context uc,
@@ -68,6 +69,10 @@ uim_create_context(void *ptr,
   int buf_len;
   char *buf;
 
+  if (uim_siod_fatal) {
+    return NULL;
+  }
+
   uc = malloc(sizeof(*uc));
   if (!uc) {
     return NULL;
@@ -360,6 +365,12 @@ uim_set_candidate_index(uim_context uc, int nth)
   return ;
 }
 
+static void
+exit_hook(void)
+{
+  uim_siod_fatal = 1;
+}
+
 static void
 uim_init_scm()
 {
@@ -375,10 +386,13 @@ uim_init_scm()
       "-v0",
     };
 
+  /* init siod */
   siod_output = stdout;
   process_cla (7, siod_argv, 1);
   init_storage ();
   init_subrs ();
+  set_fatal_exit_hook(exit_hook);
+  /**/
   uim_init_im_subrs();
   uim_init_util_subrs();
   uim_init_anthy();
