@@ -21,12 +21,13 @@
 Display *XimServer::gDpy;
 std::map<Window, XimServer *> XimServer::gServerMap;
 
-//
+// Configuration
 int g_option_mask;
 int scr_width,scr_height;
 int host_byte_order;
 
 static int lib_uim_fd;
+static char *result;
 
 #define VERSION_NAME "uim-xim under the way! Version "PACKAGE_VERSION"\n"
 char *version_name=VERSION_NAME;
@@ -350,10 +351,30 @@ get_runtime_env()
     }
 }
 
+static void
+helper_str_parse(char *str)
+{
+    //currently do nothing.
+}
+
 static void
 helper_read_cb(int fd, int ev)
 {
-  uim_helper_read_proc();
+  char *tmp;
+
+  tmp = uim_helper_read_proc(fd);
+
+  if(!result)
+    result = strdup("");
+
+  result = (char*)realloc(result, strlen(result) + strlen(tmp)+1);
+  strcat(result, tmp);
+
+  if(uim_helper_str_terminated(result)) {
+    helper_str_parse(result);
+    free(result);
+    result = strdup("");
+  }
 }
 
 static void
